package bicliqueviz;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Collection;
import java.util.Vector;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.JTree;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.TreePath;
import javax.swing.JList;
import javax.swing.BorderFactory;
import javax.swing.border.TitledBorder;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.ToolTipManager;
import javax.swing.JOptionPane;

import java.awt.Component;
import java.io.File;
import java.awt.Color;
import org.dotuseful.ui.tree.SortedTreeNode;
import org.dotuseful.ui.tree.AutomatedTreeNode;
import no.uib.cipr.matrix.sparse.SparseVector;
import no.uib.cipr.matrix.VectorEntry;

/**
 * 
 * This class represents the tree menu in the GUI
 * 
 * @author Martin Schwartz
 *
 */
public class BicliqueTreeMenu implements TreeSelectionListener
{
	private JTree tree;
	private String sizeFile;
	private String bicliqueFile;
	private TabbedPane upperTabs;
	private TabbedPane lowerTabs;
	private VertexCommList vcList;
	private BiGraph graph;
	private boolean firstTime = true;
	private Vector<TreeSet<Community>> communitiesTree;
	private Vector<Vector<Biclique>> bicliquesTree;
	private Vector<Biclique> bicliques;
	private SortedTreeNode top;
	private int numOfCliques = 0;
	private int numOfCliquesExcl = 0;
	private int maxASize = 0;
	private int maxBSize = 0;
	private final int MAX_THRESHOLD = 20;
	private final String COMMS_TEMP = "comms.tmp";
	private int minThresholdA;
	private int minThresholdB;
	private int maxThresholdA;
	private int maxThresholdB;
	private BiCliqueWin win;
	private boolean isTypeASorting;
	
	/**
	 * Constructor for when files are cached already
	 * @param loader pointer to the cache file loader
	 * @param g pointer to bigraph object
	 * @param t1 pointer to upper tabbed pane in the gui
	 * @param t2 pointer to lower tabbed pane in the gui
	 * @param vcl pointer to vertex comm list object
	 * @param w pointer to GUI window object
	 */
	public BicliqueTreeMenu(CacheFilesLoader loader,
			BiGraph g, TabbedPane t1, TabbedPane t2, VertexCommList vcl,
			BiCliqueWin w)
	{
		graph = g;
		upperTabs = t1;
		lowerTabs = t2;
		vcList = vcl;
		bicliques = loader.createBicliques();
		bicliquesTree = new Vector<Vector<Biclique>>();
		communitiesTree = new Vector<TreeSet<Community>>();
		top = new SortedTreeNode("Bicliques", true, new CommTreeNodeComparer());
		loader.createTreeMenu(top, communitiesTree, bicliquesTree, bicliques);
		tree = new JTree(top);
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
		tree.addTreeSelectionListener(this);
		ToolTipManager.sharedInstance().registerComponent(tree);
		tree.setCellRenderer(new TreeMenuRenderer());
		win = w;
		isTypeASorting = true;
	}

	
	
	/**
	 * Constructor for when files are not cached
	 * @param sz Name of the .sz file
	 * @param bi Name of the .bi file
	 * @param g Pointer to the Bigraph object
	 * @param t1 Upper tabbed pane
	 * @param t2 Lower tabbed pane
	 * @param vcl pointer to vertex comm list object
	 * @param w pointer to GUI window object
	 * @param pw progress window
	 * @param minThresA min. threshold for Node A
	 * @param minThresB min. threshold for Node B
	 * @param maxThresA max. threshold for Node A
	 * @param maxThresB max. threshold for Node B
	 */
	public BicliqueTreeMenu(String sz, String bi, 
			BiGraph g, TabbedPane t1, TabbedPane t2, VertexCommList vcl,
			BiCliqueWin w, ProgWin pw, int minThresA, int minThresB,
			int maxThresA, int maxThresB) 
	{
		graph = g;
		upperTabs = t1;
		lowerTabs = t2;
		vcList = vcl;
		bicliques = new Vector<Biclique>();
		bicliquesTree = new Vector<Vector<Biclique>>();
		communitiesTree = new Vector<TreeSet<Community>>();
		top = new SortedTreeNode("Bicliques", true, new CommTreeNodeComparer());
		sizeFile = sz;
		bicliqueFile = bi;
		minThresholdA = minThresA;
		minThresholdB = minThresB;
		maxThresholdA = maxThresA;
		maxThresholdB = maxThresB;
		win = w;
    	
		readBicliqueFile(pw);
		isTypeASorting = true;
	}
	

	/**
	 * Creates menu tree
	 * @param pw progress window
	 * @param info info file
	 */
	public void createTree(ProgWin pw, InfoFile info)
	{
		createTreeMenu(top, pw, info);
		tree = new JTree(top);
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
		tree.addTreeSelectionListener(this);
		ToolTipManager.sharedInstance().registerComponent(tree);
		tree.setCellRenderer(new TreeMenuRenderer());
	}
	
	/**
	 * @return the number of bicliques in the bigraph
	 */
	public int getNumOfCliques()
	{
		return numOfCliques;
	}
	
	/**
	 * @return the number of bicliques excluding k=1,1 in the bigraph
	 */
	public int getNumOfCliquesExcl()
	{
		return numOfCliquesExcl;
	}
	
	/**
	 * @return root of tree menu
	 */
	public SortedTreeNode getRoot()
	{
		return top;
	}
	
	public boolean changeSorting()
	{
		if(isTypeASorting)
		{
			isTypeASorting = false;
			top.setComparator(new CommTreeNodeBComparer());
			DefaultTreeModel model = new DefaultTreeModel(top);
			tree.setModel(model);
		}
		else
		{
			isTypeASorting = true;
			top.setComparator(new CommTreeNodeComparer());
			DefaultTreeModel model = new DefaultTreeModel(top);
			tree.setModel(model);
		}
		
		return isTypeASorting;
	}
	
	/**
	 * @return JTree in the menu
	 */
	public JTree getJTree()
	{
		return tree;
	}
	
	/**
	 * @return all communities in tree format
	 */
	public Vector<TreeSet<Community>> getCommunitiesTree()
	{
		return communitiesTree;
	}
	
	private void readBicliqueFile(ProgWin pw)
	{
		pw.setTextAreaHeader("Reading biclique file...");
		bicliques = new Vector<Biclique>();
		pw.setValue(0);
		findMaxSizes();		
		
		try 
		{
			File ff = new File(bicliqueFile);
			double fileSize = ff.length();
			double bytesLeft = fileSize;
			BufferedReader reader = new BufferedReader(new FileReader(bicliqueFile));
			
			String line = "";
			
			//read lines from biclique file
			//format:
			//1st line: IDs of upper part (type A) of biclique
			//2nd line: IDs of lower part (type B) of biclique
			//3rd line: empty
			//repeat until end of file
			while(line != null)
			{
				line = reader.readLine();
				if(line == null)
				{
					break;
				}
				bytesLeft -= line.getBytes().length+2;
				String[] upper = line.split("\\s");
				line = reader.readLine();
				bytesLeft -= line.getBytes().length+2;
				String[] lower = line.split("\\s");
				String sizeString = "k="+ upper.length +","+ lower.length;
				int[] bicliqueUpper = new int[upper.length];
				int[] bicliqueLower = new int[lower.length];
				
				//check if the biclique is with the specified thresholds
				//if yes, add to bicliques vector
				if(upper.length >= minThresholdA && lower.length >= minThresholdB &&
						upper.length <= maxThresholdA && lower.length <= maxThresholdB)
				{
					for(int i = 0; i < upper.length; i++)
					{
						bicliqueUpper[i] = Integer.parseInt(upper[i]);
					}
					
					for(int i = 0; i < lower.length; i++)
					{
						bicliqueLower[i] = Integer.parseInt(lower[i]);
					}
					
					int id = bicliques.size();
					Biclique bn = new Biclique(id, bicliqueUpper, bicliqueLower, sizeString);
					bicliques.add(bn);
				}

				line = reader.readLine();
				bytesLeft -= line.getBytes().length+2;
				
				//update progress bar
				if(bytesLeft > 0)
				{
					double progress = 100.0*(fileSize-bytesLeft)/fileSize;
					int p = Math.min((int)progress, 100);
					pw.setValue(p);
				}
				else
				{
					pw.setValue(100);
				}
				
				
			}
			reader.close();
			
			
		} 
		catch (FileNotFoundException e) 
		{
			e.printStackTrace();
		}
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	private void createTreeMenu(SortedTreeNode top, ProgWin pw, InfoFile info)
	{
		pw.setTextAreaHeader("Creating tree menu...");
		pw.setValue(0);
		File tmpFile = new File(COMMS_TEMP);
		double fileSize = tmpFile.length();
		double bytesLeft = fileSize;
		int id = 0;
		
		try 
		{
			BufferedReader reader = new BufferedReader(new FileReader(COMMS_TEMP));
			
			String line = "";
			
			//read lines of temporary community file
			while(line != null)
			{
				AutomatedTreeNode n = null;
				
				Vector<Biclique> bbcs = new Vector<Biclique>();
				TreeSet<Community> ccs = new TreeSet<Community>(new CommunityComparer());
				
				line = reader.readLine();
				
				if(line == null)
				{
					break;
				}
				bytesLeft -= line.getBytes().length+2;
				
				line = line.replace("(", "");
				line = line.replace(")", "");
				String[] pair = line.split(",");
				NumberPair np = new NumberPair(Integer.parseInt(pair[0]), Integer.parseInt(pair[1]));
				CommunityTreeNode ctn = new CommunityTreeNode(np.getA(),np.getB());
				n = new AutomatedTreeNode(ctn);
				
				line = reader.readLine();
				bytesLeft -= line.getBytes().length+2;
				int numOfComms = Integer.parseInt(line);
				
				//go through all communities of type k=a,b
				for(int i = 0; i < numOfComms; i++)
				{
					Vector<Biclique> bcs = new Vector<Biclique>();
					
					line = reader.readLine();
					bytesLeft -= line.getBytes().length+2;
					String[] splitLine = line.split("\\s");
					
					for(int j = 0; j < splitLine.length; j++)
					{
						Biclique b = bicliques.get(Integer.parseInt(splitLine[j]));
						bcs.add(b);
						bbcs.add(b);
					}
					
					Community c = new Community(bcs, id, n.toString());
					id++;
					c.incrementVertexOccurs(info);
					ccs.add(c);
				}
				line = reader.readLine();
				bytesLeft -= line.getBytes().length+2;
				
				Iterator<Community> itt = ccs.iterator();
				while(itt.hasNext())
				{
					n.add(new AutomatedTreeNode(itt.next()));
				}

				bicliquesTree.add(bbcs);
				communitiesTree.add(ccs);
				top.add(n);
				if(bytesLeft > 0)
				{
					double progress = 100.0*(fileSize-bytesLeft)/fileSize;
					int p = Math.min((int)progress, 100);
					pw.setValue(p);
				}
				else
				{
					pw.setValue(100);
				}
			}
			reader.close();
			
			
			tmpFile.delete();
		} 
		catch (FileNotFoundException e) 
		{
			System.err.println("Error creating menu: "+ COMMS_TEMP +" missing");
			System.exit(1);
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	private void findMaxSizes()
	{
		try 
		{
			BufferedReader reader = new BufferedReader(new FileReader(sizeFile));
			
			String line = reader.readLine();
			
			while(line != null)
			{
				String[] split = line.split("\\s");
				maxASize = Math.min(Math.max(Integer.parseInt(split[0]), maxASize), MAX_THRESHOLD);
				maxBSize = Math.min(Math.max(Integer.parseInt(split[1]), maxBSize), MAX_THRESHOLD);
				
				if(!(Integer.parseInt(split[0]) == 1 && Integer.parseInt(split[1]) == 1))
				{
					numOfCliquesExcl++;
				}
				numOfCliques++;
				line = reader.readLine();
			}
		} 
		catch (FileNotFoundException e) 
		{
			e.printStackTrace();
		}
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * @return all bicliques in tree format
	 */
	public Vector<Vector<Biclique>> getBicliquesTree()
	{
		return bicliquesTree;
	}
	
	/**
	 * @return all bicliques
	 */
	public Vector<Biclique> getBicliques()
	{
		return bicliques;
	}
	
	/**
	 * Creates a window with the community network of the selected fixed k=a,b
	 */
	public void createCommunityNetwork()
	{
		if(tree.getSelectionPath() != null)
		{
			new CommunityGraph(this, win.getInfoFile());
		}
	}
	
	/**
	 * Creates overlap matrix and returns all the communities in the bigraph
	 * @param pw progress window
	 */
	public void createOverlapMatrix(ProgWin pw)
	{	
		long t1 = System.currentTimeMillis();
		pw.setTextAreaHeader("Creating overlap matrix...");
		pw.setValue(0);
		
		//Eligible diagonals
		NumberPair[] diagonals = new NumberPair[bicliques.size()];
		
		for(int i = 0; i < diagonals.length; i++)
		{
			diagonals[i] = null;
		}
		
		Vector<IndexVectorPair> upperPairs = new Vector<IndexVectorPair>();
		Vector<IndexVectorPair> lowerPairs = new Vector<IndexVectorPair>();
		
		for(int i = 0; i < bicliques.size(); i++)
		{
			int[] upper = bicliques.get(i).getBicliqueUpper();
			int[] lower = bicliques.get(i).getBicliqueLower();
			
			int j = i;
			HashSet<Integer> currentUpperSet = new HashSet<Integer>();
			HashSet<Integer> currentLowerSet = new HashSet<Integer>();
			
			for(int k = 0; k < upper.length; k++)
			{
				currentUpperSet.add(new Integer(upper[k]));
			}
			for(int k = 0; k < lower.length; k++)
			{
				currentLowerSet.add(new Integer(lower[k]));
			}
			
			Vector<NumberPair> upperCounts = new Vector<NumberPair>();
			Vector<NumberPair> lowerCounts = new Vector<NumberPair>();
			
			if(upper.length >= minThresholdA && lower.length >= minThresholdB)
			{
				if(diagonals[i] == null)
				{
					diagonals[i] = new NumberPair(upper.length,lower.length);
				}
				upperCounts.add(new NumberPair(i, upper.length));
				lowerCounts.add(new NumberPair(i, lower.length));
			}
			
			j++;
			
			while(j < bicliques.size())
			{
				int[] upper2 = bicliques.get(j).getBicliqueUpper();
				int[] lower2 = bicliques.get(j).getBicliqueLower();
				
				if(upper2.length >= minThresholdA && lower2.length >= minThresholdB)
				{
					if(diagonals[j] == null)
					{
						diagonals[j] = new NumberPair(upper2.length,lower2.length);
					}
				}
				
				int upperCount = 0;
				int lowerCount = 0;
				
				//Number of overlaps in upper part
				for(int k = 0; k < upper2.length; k++)
				{
					if(currentUpperSet.contains(new Integer(upper2[k])))
					{
						upperCount++;
					}
				}
				
				//Number of overlaps in lower part
				for(int k = 0; k < lower2.length; k++)
				{
					if(currentLowerSet.contains(new Integer(lower2[k])))
					{
						lowerCount++;
					}
				}
				
				if(upperCount >= minThresholdA-1 && lowerCount >= minThresholdB-1)
				{
					if(upperCount > 0 && lowerCount > 0 ||
							upperCount > 0 && lowerCount == 0 ||
							upperCount == 0 && lowerCount > 0)
					{
						if(upperCount > 0)
						{
							upperCounts.add(new NumberPair(j, upperCount));
						}
						
						if(lowerCount > 0)
						{
							lowerCounts.add(new NumberPair(j, lowerCount));
						}
					}
				}
				
				j++;
			}
			
			if(upperCounts.size() > 0)
			{
				SparseVector usv = new SparseVector(bicliques.size(), upperCounts.size());
				
				for(int k = 0; k < upperCounts.size(); k++)
				{
					NumberPair np = upperCounts.get(k);
					usv.set(np.getA(), np.getB());
				}
				upperPairs.add(new IndexVectorPair(i, usv));
			}
			
			if(lowerCounts.size() > 0)
			{
				SparseVector lsv = new SparseVector(bicliques.size(), lowerCounts.size());
				
				for(int k = 0; k < lowerCounts.size(); k++)
				{
					NumberPair np = lowerCounts.get(k);
					lsv.set(np.getA(), np.getB());
				}
				lowerPairs.add(new IndexVectorPair(i, lsv));
			}
			
			double k = i;
			double l = bicliques.size();
			double progress = 100.0*k/l;
			int p = Math.min((int)progress, 100);
			pw.setValue(p);
		}
		
		long t2 = System.currentTimeMillis();
		System.out.println("Overlap: "+(t2-t1));
		try 
		{
			BufferedWriter writer = new BufferedWriter(new FileWriter("times.txt", true));
			writer.write("Overlap: "+(t2-t1)+"\n");
			writer.close();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
		
		pw.setTextAreaHeader("Creating communities...");
		pw.setValue(0);
		
		System.out.println("first done");
		System.out.println("sizes: "+upperPairs.size()+","+lowerPairs.size());
		
		for(int a = 1; a <= maxASize; a++)
		{
			for(int b = 1; b <= maxBSize; b++)
			{
				if(a == 1 && b == 1)
				{
					b = 2;
				}
				System.out.println(a+","+b);
				Vector<HashSet<Integer>> com = new Vector<HashSet<Integer>>();
	
				for(int i = 0; i < upperPairs.size(); i++)
				{
					SparseVector upperSv = upperPairs.get(i).getVector();
					SparseVector lowerSv = lowerPairs.get(i).getVector();
					
					if(upperSv.get(i) >= a && lowerSv.get(i) >= b)
					{
						Iterator<VectorEntry> upperIt = upperSv.iterator();
						Iterator<VectorEntry> lowerIt = lowerSv.iterator();
						
						while(upperIt.hasNext() || lowerIt.hasNext())
						{
							VectorEntry upperEntry = null;
							VectorEntry lowerEntry = null;
							
							if(upperIt.hasNext())
							{
								upperEntry = upperIt.next();
							}
							
							if(lowerIt.hasNext())
							{
								lowerEntry = lowerIt.next();
							}
							
							if(upperEntry != null && lowerEntry != null)
							{
								int upperIndex = upperEntry.index();
								int lowerIndex = lowerEntry.index();
								
								if(upperIndex == lowerIndex)
								{
									NumberPair np = diagonals[upperIndex];
									
									if(np != null)
									{
										if(np.getA() >= a && np.getB() >= b &&
												upperEntry.get() >= a-1 &&
												lowerEntry.get() >= b-1)
										{
											int  j = upperIndex; //and = lowerIndex
											if(com.size() > 0)
											{
												boolean found = false;
												for(int c = 0; c < com.size(); c++)
												{
													if(com.get(c).contains(i) || com.get(c).contains(j))
													{
														boolean found2 = false;
														for(int d = c+1; d < com.size(); d++)
														{
															if(com.get(d).contains(i) || com.get(d).contains(j))
															{
																HashSet<Integer> set = com.remove(d);
																set.add(i);
																set.add(j);
																com.get(c).addAll(set);
																found2 = true;
																break;
															}
														}
														
														if(!found2)
														{
															com.get(c).add(i);
															com.get(c).add(j);
														}
														found = true;
														break;
													}
												}
												
												if(!found)
												{
													HashSet<Integer> set = new HashSet<Integer>();
													set.add(i);
													set.add(j);
													com.add(set);
												}
											}
											else
											{
												HashSet<Integer> set = new HashSet<Integer>();
												set.add(i);
												set.add(j);
												com.add(set);
											}
										}
									}	
								}
								else
								{
									NumberPair upperNP = diagonals[upperIndex];
									
									if(upperNP != null)
									{
										if(upperNP.getA() >= a && upperNP.getB() >= b &&
												upperEntry.get() >= a-1 &&
												lowerSv.get(upperIndex) >= b-1)
										{
											int  j = upperIndex;
											if(com.size() > 0)
											{
												boolean found = false;
												for(int c = 0; c < com.size(); c++)
												{
													if(com.get(c).contains(i) || com.get(c).contains(j))
													{
														boolean found2 = false;
														for(int d = c+1; d < com.size(); d++)
														{
															if(com.get(d).contains(i) || com.get(d).contains(j))
															{
																HashSet<Integer> set = com.remove(d);
																set.add(i);
																set.add(j);
																com.get(c).addAll(set);
																found2 = true;
																break;
															}
														}
														
														if(!found2)
														{
															com.get(c).add(i);
															com.get(c).add(j);
														}
														found = true;
														break;
													}
												}
												
												if(!found)
												{
													HashSet<Integer> set = new HashSet<Integer>();
													set.add(i);
													set.add(j);
													com.add(set);
												}
											}
											else
											{
												HashSet<Integer> set = new HashSet<Integer>();
												set.add(i);
												set.add(j);
												com.add(set);
											}
										}
									}
									
									NumberPair lowerNP = diagonals[lowerIndex];
									
									if(lowerNP != null)
									{
										if(lowerNP.getA() >= a && lowerNP.getB() >= b &&
												upperSv.get(lowerIndex) >= a-1 &&
												lowerEntry.get() >= b-1)
										{
											int  j = lowerIndex;
											if(com.size() > 0)
											{
												boolean found = false;
												for(int c = 0; c < com.size(); c++)
												{
													if(com.get(c).contains(i) || com.get(c).contains(j))
													{
														boolean found2 = false;
														for(int d = c+1; d < com.size(); d++)
														{
															if(com.get(d).contains(i) || com.get(d).contains(j))
															{
																HashSet<Integer> set = com.remove(d);
																set.add(i);
																set.add(j);
																com.get(c).addAll(set);
																found2 = true;
																break;
															}
														}
														
														if(!found2)
														{
															com.get(c).add(i);
															com.get(c).add(j);
														}
														found = true;
														break;
													}
												}
												
												if(!found)
												{
													HashSet<Integer> set = new HashSet<Integer>();
													set.add(i);
													set.add(j);
													com.add(set);
												}
											}
											else
											{
												HashSet<Integer> set = new HashSet<Integer>();
												set.add(i);
												set.add(j);
												com.add(set);
											}
										}
									}
								}
							}
							else if(upperEntry != null && lowerEntry == null)
							{
								int upperIndex = upperEntry.index();
								
								NumberPair upperNP = diagonals[upperIndex];
								
								if(upperNP != null)
								{
									if(upperNP.getA() >= a && upperNP.getB() >= b &&
											upperEntry.get() >= a-1 &&
											lowerSv.get(upperIndex) >= b-1)
									{
										int  j = upperIndex;
										if(com.size() > 0)
										{
											boolean found = false;
											for(int c = 0; c < com.size(); c++)
											{
												if(com.get(c).contains(i) || com.get(c).contains(j))
												{
													boolean found2 = false;
													for(int d = c+1; d < com.size(); d++)
													{
														if(com.get(d).contains(i) || com.get(d).contains(j))
														{
															HashSet<Integer> set = com.remove(d);
															set.add(i);
															set.add(j);
															com.get(c).addAll(set);
															found2 = true;
															break;
														}
													}
													
													if(!found2)
													{
														com.get(c).add(i);
														com.get(c).add(j);
													}
													found = true;
													break;
												}
											}
											
											if(!found)
											{
												HashSet<Integer> set = new HashSet<Integer>();
												set.add(i);
												set.add(j);
												com.add(set);
											}
										}
										else
										{
											HashSet<Integer> set = new HashSet<Integer>();
											set.add(i);
											set.add(j);
											com.add(set);
										}
									}
								}
							}
							else if(upperEntry == null && lowerEntry != null)
							{
								int lowerIndex = lowerEntry.index();
								NumberPair lowerNP = diagonals[lowerIndex];
								
								if(lowerNP != null)
								{
									if(lowerNP.getA() >= a && lowerNP.getB() >= b &&
											upperSv.get(lowerIndex) >= a-1 &&
											lowerEntry.get() >= b-1)
									{
										int  j = lowerIndex;
										if(com.size() > 0)
										{
											boolean found = false;
											for(int c = 0; c < com.size(); c++)
											{
												if(com.get(c).contains(i) || com.get(c).contains(j))
												{
													boolean found2 = false;
													for(int d = c+1; d < com.size(); d++)
													{
														if(com.get(d).contains(i) || com.get(d).contains(j))
														{
															HashSet<Integer> set = com.remove(d);
															set.add(i);
															set.add(j);
															com.get(c).addAll(set);
															found2 = true;
															break;
														}
													}
													
													if(!found2)
													{
														com.get(c).add(i);
														com.get(c).add(j);
													}
													found = true;
													break;
												}
											}
											
											if(!found)
											{
												HashSet<Integer> set = new HashSet<Integer>();
												set.add(i);
												set.add(j);
												com.add(set);
											}
										}
										else
										{
											HashSet<Integer> set = new HashSet<Integer>();
											set.add(i);
											set.add(j);
											com.add(set);
										}
									}
								}
							}
						}
					}
				}
				
				Vector<Collection<Integer>> cs = new Vector<Collection<Integer>>();
				
				Iterator<HashSet<Integer>> it3 = com.iterator();
				
				while(it3.hasNext())
				{
					Set<Integer> gg = it3.next();
					if(gg.size() > 1)
					{
						cs.add(gg);
					}
				}
				
				if(cs.size() > 0)
				{
					try 
					{
						BufferedWriter writer = new BufferedWriter(new FileWriter(COMMS_TEMP, true));
						
						NumberPair np = new NumberPair(a,b);
						writer.write(np.toString()+"\n");
						writer.write(cs.size()+"\n");
						
						for(int i = 0; i < cs.size(); i++)
						{
							Integer[] ii = cs.get(i).toArray(new Integer[0]);
							String s = "";
							for(int j = 0; j < ii.length; j++)
							{
								s += ii[j]+"\t";
							}
							s = s.trim();
							writer.write(s+"\n");
						}
						writer.write("\n");
						writer.close();
					} 
					catch (IOException e) 
					{
						e.printStackTrace();
					}
				}
			}
		}
	}
	
	public void createOverlapMatrix2(ProgWin pw)
	{	
		long t1 = System.currentTimeMillis();
		pw.setTextAreaHeader("Creating overlap matrix...");
		pw.setValue(0);
		
		//Eligible diagonals
		NumberPair[] diagonals = new NumberPair[bicliques.size()];
		
		for(int i = 0; i < diagonals.length; i++)
		{
			diagonals[i] = null;
		}
		
		//Matrix m[a][b] = vector of all overlapping bicliques and
		//that pass the (a+1,b+1) threshold
		Vector<NumberPair>[][] abMatrix = new Vector[maxASize][maxBSize];
		
		for(int i = 0; i < maxASize; i++)
		{
			for(int j = 0; j < maxBSize; j++)
			{
				abMatrix[i][j] = null;
			}
		}
		
		for(int i = 0; i < bicliques.size(); i++)
		{
			int[] upper = bicliques.get(i).getBicliqueUpper();
			int[] lower = bicliques.get(i).getBicliqueLower();
			
			int j = i;
			HashSet<Integer> currentUpperSet = new HashSet<Integer>();
			HashSet<Integer> currentLowerSet = new HashSet<Integer>();
			
			for(int k = 0; k < upper.length; k++)
			{
				currentUpperSet.add(new Integer(upper[k]));
			}
			for(int k = 0; k < lower.length; k++)
			{
				currentLowerSet.add(new Integer(lower[k]));
			}
			
			//check diagonal to make sure we only include bicliques of correct size
			if(upper.length >= minThresholdA && lower.length >= minThresholdB)
			{
				if(diagonals[i] == null)
				{
					diagonals[i] = new NumberPair(upper.length,lower.length);
				}
			}
			
			j++;
			
			while(j < bicliques.size())
			{
				int[] upper2 = bicliques.get(j).getBicliqueUpper();
				int[] lower2 = bicliques.get(j).getBicliqueLower();
				
				if(upper2.length >= minThresholdA && lower2.length >= minThresholdB)
				{
					if(diagonals[j] == null)
					{
						diagonals[j] = new NumberPair(upper2.length,lower2.length);
					}
				}
				
				int upperCount = 0;
				int lowerCount = 0;
				
				//Number of overlaps in upper part
				for(int k = 0; k < upper2.length; k++)
				{
					if(currentUpperSet.contains(new Integer(upper2[k])))
					{
						upperCount++;
					}
				}
				
				//Number of overlaps in lower part
				for(int k = 0; k < lower2.length; k++)
				{
					if(currentLowerSet.contains(new Integer(lower2[k])))
					{
						lowerCount++;
					}
				}
				
				//Remaining off diagonal bicliques must be great than or equal to a-1,b-1
				if(upperCount >= minThresholdA-1 && lowerCount >= minThresholdB-1)
				{
					if(upperCount > 0 && lowerCount > 0 ||
							upperCount > 0 && lowerCount == 0 ||
							upperCount == 0 && lowerCount > 0)
					{
						NumberPair ijPair = new NumberPair(upperCount,lowerCount);
						NumberPair jjPair = new NumberPair(upper2.length,lower2.length);
						
						if(diagonals[j] != null)
						{
							int aMax = Math.min(ijPair.getA()+1, jjPair.getA());
							int bMax = Math.min(ijPair.getB()+1, jjPair.getB());
							
							aMax = Math.min(aMax, MAX_THRESHOLD);
							bMax = Math.min(bMax, MAX_THRESHOLD);
							
							int maxA = 0;
							int maxB = 0;
							for(int a = 1; a <= aMax; a++)
							{
								for(int b = 1; b <= bMax; b++)
								{
									if(a == 1 && b == 1)
									{
										b = 2;
									}
									
									if(upper.length >= a && lower.length >= b &&
											upper2.length >= a && lower2.length >= b &&
											upperCount >= a-1 && lowerCount >= b-1)
									{
										if(abMatrix[a-1][b-1] == null)
										{
											abMatrix[a-1][b-1] = new Vector<NumberPair>();
										}
	
										abMatrix[a-1][b-1].add(new NumberPair(i,j));
										
										maxA = Math.max(a-1, maxA);
										maxB = Math.max(b-1, maxB);
									}
								}
								
								System.out.println("("+i+","+j+") "+upper.length+"\t"+lower.length+"\t"+upper2.length+"\t"+lower2.length+"\t"+maxA+"\t"+maxB);
							}
							System.out.println("final ("+i+","+j+") "+upper.length+"\t"+lower.length+"\t"+upper2.length+"\t"+lower2.length+"\t"+upperCount+"\t"+lowerCount+"\n");
						}
					}
				}
				
				j++;
			}
			double k = i;
			double l = bicliques.size();
			double progress = 100.0*k/l;
			int p = Math.min((int)progress, 100);
			pw.setValue(p);
		}
		
		long t2 = System.currentTimeMillis();
		System.out.println("Overlap: "+(t2-t1));
		try 
		{
			BufferedWriter writer = new BufferedWriter(new FileWriter("times.txt", true));
			writer.write("Overlap: "+(t2-t1)+"\n");
			writer.close();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}

		diagonals = null;
		
		pw.setTextAreaHeader("Creating communities...");
		pw.setValue(0);
		
		double kk = 0;
		double l = maxASize*maxBSize;
		
		System.out.println("first done");
		
//		method 2
		for(int a = 0; a < maxASize; a++)
		{
			for(int b = 0; b < maxBSize; b++)
			{
				long free = Runtime.getRuntime().totalMemory()-Runtime.getRuntime().freeMemory();
				System.out.println("Free Memory: "+free);
				double progress = 100.0*((double)kk)/l;
				int p = Math.min((int)progress, 100);
				pw.setValue(p);
				kk++;
				if(abMatrix[a][b] != null)
				{
					Vector<HashSet<Integer>> com = new Vector<HashSet<Integer>>();
					
					for(int k = 0; k < abMatrix[a][b].size(); k++)
					{
						System.out.println(k+"/"+abMatrix[a][b].size() +" ("+a+","+b+")");
						
						int i = abMatrix[a][b].get(k).getA();
						int j = abMatrix[a][b].get(k).getB();
						
						abMatrix[a][b].set(k, null);
						
						if(com.size() > 0)
						{
							boolean found = false;
							for(int c = 0; c < com.size(); c++)
							{
								if(com.get(c).contains(i) || com.get(c).contains(j))
								{
									boolean found2 = false;
									for(int d = c+1; d < com.size(); d++)
									{
										if(com.get(d).contains(i) || com.get(d).contains(j))
										{
											HashSet<Integer> set = com.remove(d);
											set.add(i);
											set.add(j);
											com.get(c).addAll(set);
											found2 = true;
											break;
										}
									}
									
									if(!found2)
									{
										com.get(c).add(i);
										com.get(c).add(j);
									}
									found = true;
									break;
								}
							}
							
							if(!found)
							{
								HashSet<Integer> set = new HashSet<Integer>();
								set.add(i);
								set.add(j);
								com.add(set);
							}
						}
						else
						{
							HashSet<Integer> set = new HashSet<Integer>();
							set.add(i);
							set.add(j);
							com.add(set);
						}
					}
					
					Vector<Collection<Integer>> cs = new Vector<Collection<Integer>>();
					
					Iterator<HashSet<Integer>> it3 = com.iterator();
					
					while(it3.hasNext())
					{
						Set<Integer> gg = it3.next();
						if(gg.size() > 1)
						{
							cs.add(gg);
						}
					}
					
					if(cs.size() > 0)
					{
						try 
						{
							BufferedWriter writer = new BufferedWriter(new FileWriter(COMMS_TEMP, true));
							
							NumberPair np = new NumberPair(a+1,b+1);
							writer.write(np.toString()+"\n");
							writer.write(cs.size()+"\n");
							
							for(int i = 0; i < cs.size(); i++)
							{
								Integer[] ii = cs.get(i).toArray(new Integer[0]);
								String s = "";
								for(int j = 0; j < ii.length; j++)
								{
									s += ii[j]+"\t";
								}
								s = s.trim();
								writer.write(s+"\n");
							}
							writer.write("\n");
							writer.close();
						} 
						catch (IOException e) 
						{
							e.printStackTrace();
						}
					}
				}
				abMatrix[a][b] = null;
			}
		}
	}
	
	public void valueChanged(TreeSelectionEvent e)
	{
		if(firstTime)
		{
			firstTime = false;
			graph.getVV().setVisible(true);
		}
		vcList.clearList();
		graph.getVV().getPickedVertexState().clear();
    	graph.getVV().getPickedEdgeState().clear();

		if(upperTabs.getBicliquesList() != null)
		{
			if(upperTabs.getBicliquesList().getSelectedValues().length > 0)
			{
				upperTabs.getBicliquesList().clearSelection();
				lowerTabs.getBicliquesList().clearSelection();
				graph.getVV().getPickedVertexState().clear();
				graph.getVV().getPickedEdgeState().clear();
			}
		}

		TreePath[] paths = e.getPaths();

		if (paths == null)
		{
			return;
		}
		
		for(int i = 0; i < paths.length; i++)
		{
			//System.out.println(paths[i]);
			DefaultMutableTreeNode node = 
				(DefaultMutableTreeNode)paths[i].getLastPathComponent();
			
			if(node.isLeaf()) 
			{
				Community comm = (Community)node.getUserObject();
				if(e.isAddedPath(i))
				{
					boolean allowed = true;
					
					if(comm.getNumOfTotalNodes() >= BiCliqueWin.WARNING_NODESIZE)
					{
						int choice = JOptionPane.showConfirmDialog(win,"Are you sure you wish\n to open the following community?\n" +
								"It contains more than "+BiCliqueWin.WARNING_NODESIZE+" nodes", "Large community", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
						
						if(choice == JOptionPane.NO_OPTION)
						{
							allowed = false;
						}
					}
					
					if(allowed)
					{
						graph.changeGraph(graph.createBiclique(comm.getBicliques()));
						Collection<VertexObj> nodes = graph.getGraph().getVertices();
						Iterator<VertexObj> it = nodes.iterator();
						Vector<VertexObj> vas = new Vector<VertexObj>();
						Vector<VertexObj> vbs = new Vector<VertexObj>();
						while(it.hasNext())
						{
							VertexObj v = it.next();
							
							if(v.isVertexA())
							{
								vas.add(v);
							}
							else
							{
								vbs.add(v);
							}
						}
						
						graph.setVertexLocations(vas.size(), vbs.size());
						
						JList verticesA = new JList(vas);
						JList verticesB = new JList(vbs);
						
						verticesA.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), "Vertices", TitledBorder.CENTER, TitledBorder.TOP));
						verticesB.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), "Vertices", TitledBorder.CENTER, TitledBorder.TOP));
						
						Vector<EdgeObj> list = new Vector<EdgeObj>(graph.getGraph().getEdges());
						JList edgesA = new JList(list);
						JList edgesB = new JList(list);
						
						edgesA.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), "Edges", TitledBorder.CENTER, TitledBorder.TOP));
						edgesB.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), "Edges", TitledBorder.CENTER, TitledBorder.TOP));
						
						JList bicliquesA = new JList(comm.getBicliques());
						JList bicliquesB = new JList(comm.getBicliques());
						
						bicliquesA.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), "Bicliques", TitledBorder.CENTER, TitledBorder.TOP));
						bicliquesB.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), "Bicliques", TitledBorder.CENTER, TitledBorder.TOP));
						
						upperTabs.addTab(comm.getID() +". ("+ node.getParent().toString() +")", verticesA, edgesA, bicliquesA, graph);
						lowerTabs.addTab(comm.getID() +". ("+ node.getParent().toString() +")", verticesB, edgesB, bicliquesB, graph);
					}
					else
					{
						tree.removeSelectionPath(paths[i]);
					}
				}
				else
				{
					upperTabs.removeTab(comm.getID() +". ("+ node.getParent().toString() +")");
					lowerTabs.removeTab(comm.getID() +". ("+ node.getParent().toString() +")");
					
					if(upperTabs.getTabbedPane().getTabCount() == 0)
					{
						graph.clearGraph();
					}
				}
			}
		}
	}
	
    private class TreeMenuRenderer extends DefaultTreeCellRenderer 
    {
    	static final long serialVersionUID = 52341;

        public TreeMenuRenderer() 
        {

        }

        public Component getTreeCellRendererComponent(
                            JTree tree,
                            Object value,
                            boolean sel,
                            boolean expanded,
                            boolean leaf,
                            int row,
                            boolean hasFocus) 
        {
            super.getTreeCellRendererComponent(
                            tree, value, sel,
                            expanded, leaf, row,
                            hasFocus);
            
            if (leaf) 
            {
                setToolTipText(value.toString());
            } 
            else 
            {
                setToolTipText(null); //no tool tip
            } 

            return this;
        }
    }

}
