package bicliqueviz;

import java.awt.Color;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;
import java.util.TreeSet;
import java.util.TreeMap;

import org.dotuseful.ui.tree.SortedTreeNode;
import org.dotuseful.ui.tree.AutomatedTreeNode;

/**
 * 
 * This class is used to load graphs from cache files
 * 
 * @author Martin Schwartz
 *
 */
public class CacheFilesLoader 
{
	private final String BICLIQUES;
	private ProgWin pw;
	private File directory;
	private final String COMM_BICLIQUES = "communities_bicliques";
	private final String VERTICES;

	/**
	 * @param dir directory for the cached files
	 * @param p progress window
	 */
	public CacheFilesLoader(String dir, ProgWin p) 
	{
		directory = new File(dir);
		BICLIQUES = dir+"/bicliques";
		VERTICES = dir+"/vertices";
		pw = p;
	}
	
	/**
	 * Initialise cache file loader
	 * @param win pointer to the GUI window
	 */
	public void init(BiCliqueWin win)
	{
		createVertices(win.getInfoFile().getIDToInfo());
	}
	
	private void createVertices(Hashtable<IDNodeTypePair, VertexObj> infoFile)
	{
		pw.setTextAreaHeader("Creating vertices...");
		pw.setValue(0);
		
		try 
		{
			File ff = new File(VERTICES);
			double fileSize = ff.length();
			double bytesLeft = fileSize;
			BufferedReader reader = new BufferedReader(new FileReader(VERTICES));
			
			String line = reader.readLine();
			
			while(line != null)
			{
				bytesLeft -= line.getBytes().length+2;
				String[] splitLine = line.split("\\s", 2);
				String[] v_args = splitLine[1].split("\t");
				
				int a = Integer.parseInt(splitLine[0]);
				String name = v_args[0];
				boolean vertexA = v_args[1].equals("true");
				int occurs = Integer.parseInt(v_args[2]);
				String[] rgb = v_args[3].split(",");
				Color c = new Color(Integer.parseInt(rgb[0]),Integer.parseInt(rgb[1]),Integer.parseInt(rgb[2]));
				String shape = v_args[4];
				VertexObj v = new VertexObj(name, a, vertexA, c, shape);
				v.setCliqueOccurs(occurs);
				IDNodeTypePair pa = new IDNodeTypePair(v.getID(),v.isVertexA());
				
				if(!infoFile.containsKey(pa))
				{
					infoFile.put(pa, v);
				}
				
				line = reader.readLine();
				
				if(bytesLeft > 0)
				{
					double progress = 100.0*(fileSize-bytesLeft)/fileSize;
					int p = Math.min((int)progress, 100);
					pw.setValue(p);
				}
				else
				{
					pw.setValue(100);
				}
			}
			reader.close();
		} 
		catch (FileNotFoundException e) 
		{
			e.printStackTrace();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}

	/**
	 * @return a vector with all the bicliques
	 */
	public Vector<Biclique> createBicliques()
	{
		Vector<Biclique> bicliques = new Vector<Biclique>();
		pw.setTextAreaHeader("Reading biclique file...");
		pw.setValue(0);
		
		try 
		{
			File ff = new File(BICLIQUES);
			double fileSize = ff.length();
			double bytesLeft = fileSize;
			BufferedReader reader = new BufferedReader(new FileReader(BICLIQUES));
			
			String line = "";
			
			while(line != null)
			{
				line = reader.readLine();
				if(line == null)
				{
					break;
				}
				bytesLeft -= line.getBytes().length+2;
				String[] upper = line.split("\\s");
				
				line = reader.readLine();
				bytesLeft -= line.getBytes().length+2;
				String[] lower = line.split("\\s");
				String sizeString = "k="+ (upper.length-1) +","+ (lower.length-1);
				int[] bicliqueUpper = new int[upper.length-1];
				int[] bicliqueLower = new int[lower.length-1];
				
				for(int i = 1; i < upper.length; i++)
				{
					bicliqueUpper[i-1] = Integer.parseInt(upper[i]);
				}
				
				for(int i = 1; i < lower.length; i++)
				{
					bicliqueLower[i-1] = Integer.parseInt(lower[i]);
				}
				
				int id = bicliques.size();
				Biclique bn = new Biclique(id, bicliqueUpper, bicliqueLower, sizeString);
				bicliques.add(bn);

				if(bytesLeft > 0)
				{
					double progress = 100.0*(fileSize-bytesLeft)/fileSize;
					int p = Math.min((int)progress, 100);
					pw.setValue(p);
				}
				else
				{
					pw.setValue(100);
				}
			}
			reader.close();
		} 
		catch (FileNotFoundException e) 
		{
			e.printStackTrace();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
		
		return bicliques;
	}
	
	/**
	 * Creates the tree menu
	 * @param top top node of the tree menu
	 * @param communitiesTree pointer to the community tree
	 * @param bicliquesTree pointer to the biclique tree
	 * @param bicliques the vector with the bicliques
	 */
	public void createTreeMenu(SortedTreeNode top,
			Vector<TreeSet<Community>> communitiesTree,
			Vector<Vector<Biclique>> bicliquesTree,
			Vector<Biclique> bicliques)
	{
		pw.setTextAreaHeader("Creating tree menu...");
		File[] dirs = directory.listFiles();
		TreeMap<NumberPair, File> files = new TreeMap<NumberPair, File>(new NumberPairComparer());
		
		double numOfKeys = dirs.length;
		double k = 0;
		
		int id = 0;
		
		for(int i = 0; i < dirs.length; i++)
		{
			if(dirs[i].isDirectory())
			{
				String s = dirs[i].getName();
				s = s.replace("k=", "");
				String[] split = s.split(",");
				NumberPair np = new NumberPair(Integer.parseInt(split[0]), Integer.parseInt(split[1]));
				files.put(np, dirs[i]);
			}
		}
		
		Iterator<NumberPair> it = files.keySet().iterator();
		
		while(it.hasNext())
		{
			NumberPair np = it.next();
			File d = files.get(np);
			
			CommunityTreeNode ctn = new CommunityTreeNode(np.getA(),np.getB());
			AutomatedTreeNode n = 
				new AutomatedTreeNode(ctn);
			
			File commBicliques = new File(d, COMM_BICLIQUES);
			
			Vector<Biclique> bbcs = new Vector<Biclique>();
			TreeSet<Community> cs = new TreeSet<Community>(new CommunityComparer());
			
			try 
			{
				BufferedReader reader = new BufferedReader(new FileReader(commBicliques));
				
				String line = reader.readLine();
				
				while(line != null)
				{
					Vector<Biclique> bcs = new Vector<Biclique>();
					String[] splitLine = line.split("\\s");
					
					for(int j = 1; j < splitLine.length; j++)
					{
						Biclique b = bicliques.get(Integer.parseInt(splitLine[j]));
						bcs.add(b);
						bbcs.add(b);
					}
					
					Community c = new Community(bcs, id, n.toString());
					id++;
					cs.add(c);
					
					line = reader.readLine();
				}
				
				Iterator<Community> itt = cs.iterator();
				while(itt.hasNext())
				{
					n.add(new AutomatedTreeNode(itt.next()));
				}
				
				reader.close();
			} 
			catch (FileNotFoundException e) 
			{
				e.printStackTrace();
			} 
			catch (IOException e) 
			{
				e.printStackTrace();
			}
			
			bicliquesTree.add(bbcs);
			communitiesTree.add(cs);
			top.add(n);
			double progress = 100.0*k/numOfKeys;
			int p = Math.min((int)progress, 100);
			pw.setValue(p);
			k++;
		}
	}
}
