package bicliqueviz;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.Component;
import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.PathIterator;
import java.awt.geom.GeneralPath;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Vector;
import java.util.Iterator;
import java.util.ConcurrentModificationException;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.border.TitledBorder;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JSplitPane;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.text.Position;
import javax.swing.tree.TreePath;

import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.algorithms.layout.CircleLayout;
import edu.uci.ics.jung.algorithms.layout.FRLayout;
import edu.uci.ics.jung.graph.SparseGraph;
import edu.uci.ics.jung.algorithms.layout.Layout;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.DefaultModalGraphMouse;
import edu.uci.ics.jung.visualization.control.ModalGraphMouse.Mode;
import edu.uci.ics.jung.visualization.decorators.ToStringLabeller;
import edu.uci.ics.jung.visualization.picking.PickedState;
import edu.uci.ics.jung.algorithms.layout.ISOMLayout;
import edu.uci.ics.jung.algorithms.layout.KKLayout;
import edu.uci.ics.jung.algorithms.layout.SpringLayout;
import edu.uci.ics.jung.visualization.picking.ShapePickSupport;
import edu.uci.ics.jung.visualization.VisualizationServer;
import edu.uci.ics.jung.visualization.Layer;
import edu.uci.ics.jung.graph.util.Pair;
import edu.uci.ics.jung.algorithms.util.Context;
import edu.uci.ics.screencap.Dump;
import edu.uci.ics.screencap.EPSDump;

/**
 * 
 * Class community network
 * 
 * @author Martin Schwartz
 *
 */
public class CommunityGraph implements ActionListener
{
	private final float THICKNESS_INC = 0.5f;
	private int selectedRow;
	private JList mouseOverInfo;
	private JFrame frame;
	private InfoFile infoFile;
	private JMenuItem screenshot;
	private SparseGraph<VertexObj, EdgeObj> g;
	private CustomVV<VertexObj, EdgeObj> vv;

	/**
	 * 
	 * Creates instance of community network (will be shown in a seperate window)
	 * 
	 * @param treeMenu Pointer to the tree menu
	 * @param info Pointer to the info file
	 */
	public CommunityGraph(BicliqueTreeMenu treeMenu, InfoFile info) 
	{
		createGraph(treeMenu);
		selectedRow = 0;
		infoFile = info;
	}
	
	/**
	 * 
	 * Creates the graph and window it should be shown in
	 * 
	 * @param treeMenu Tree menu
	 */
	public void createGraph(BicliqueTreeMenu treeMenu)
	{
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)treeMenu.getJTree().getSelectionPath().getLastPathComponent();
		
		if(node.toString().startsWith("k="))
		{
			g = new SparseGraph<VertexObj, EdgeObj>();
			Dimension d = new Dimension(600,600);
			vv = new CustomVV<VertexObj, EdgeObj>(new FRLayout<VertexObj, EdgeObj>(g), d);
			CircleLayout<VertexObj, EdgeObj> layout = new CircleLayout<VertexObj, EdgeObj>(g);
			
			vv.setBackground(Color.white);
			final DefaultModalGraphMouse mouse = new DefaultModalGraphMouse();
			mouse.setMode(Mode.PICKING);
			vv.setGraphMouse(mouse);
			vv.addKeyListener(mouse.getModeKeyListener());
			mouse.add(new CustomPopupPlugin2());
			
			PickedState<VertexObj> picked_state = vv.getPickedVertexState();
			PickedState<EdgeObj> picked_edge_state = vv.getPickedEdgeState();
			vv.getRenderContext().setVertexFillPaintTransformer( new VertexFillColour(picked_state));
			vv.getRenderContext().setEdgeStrokeTransformer(new EdgeStroke(picked_edge_state));
			vv.getRenderContext().setEdgeDrawPaintTransformer(new EdgeFill());
			vv.getRenderContext().setVertexShapeTransformer(new CommunitySizeShape());
			vv.getRenderContext().setVertexStrokeTransformer(new VertexStroke(picked_state));
			vv.getRenderContext().setVertexLabelTransformer(new ToStringLabeller<VertexObj>());
			vv.getRenderer().setVertexLabelRenderer(new CustomVertexLabelRenderer());
			vv.getRenderer().setVertexRenderer(new CustomVertexRenderer());
			vv.setVertexToolTipTransformer(new CommunityToolTip());
			vv.setEdgeToolTipTransformer(new CommunityEdgeToolTip());
			
			String name = node.toString();
			int i = 0;
			for(i = 0; i < treeMenu.getRoot().getChildCount(); i++)
			{
				String n = treeMenu.getRoot().getChildAt(i).toString();
				
				if(name.equals(n))
				{
					treeMenu.getJTree().expandRow(i+1);
					selectedRow = i+1;
					break;
				}
			}
			
			vv.setPickSupport(new CustomPickSupport(vv, treeMenu, vv));
			
			Community[] cs = treeMenu.getCommunitiesTree().get(i).toArray(new Community[0]);
			
			for(int j = 0; j < cs.length; j++)
			{
				Community cj = cs[j];
				VertexObj vj = new VertexObj(null, cs[j].getID(), false, new Color(0,255,0,0), "CIRCLE");
				
				if(!g.containsVertex(vj))
				{
					double uns = cj.getNumOfUpperNodes();
					double lns = cj.getNumOfLowerNodes();
					double numOfTotalNodes = uns+lns;
					vj.setUpperPercent((uns/numOfTotalNodes)*100.0);
					vj.setLowerPercent((lns/numOfTotalNodes)*100.0);
					vj.setNumOfUpperNodes((int)(uns));
					vj.setNumOfLowerNodes((int)(lns));
					g.addVertex(vj);
				}
				
				for(int k = j+1; k < cs.length; k++)
				{
					Community ck = cs[k];
					VertexObj vk = new VertexObj(null, cs[k].getID(), false, new Color(0,255,0,0), "CIRCLE");
					
					if(!g.containsVertex(vk))
					{
						double uns = ck.getNumOfUpperNodes();
						double lns = ck.getNumOfLowerNodes();
						double numOfTotalNodes = uns+lns;
						vk.setUpperPercent((uns/numOfTotalNodes)*100.0);
						vk.setLowerPercent((lns/numOfTotalNodes)*100.0);
						vk.setNumOfUpperNodes((int)(uns));
						vk.setNumOfLowerNodes((int)(lns));
						g.addVertex(vk);
					}
					
					if(!vj.equals(vk))
					{
						NumberPair np = cj.getNumOfOverlappingNodes(ck);
						
						if(np.getA() > 0)
						{
							float t = ((float)np.getA())*THICKNESS_INC + 1.0f;
							EdgeObj e = new EdgeObj(vj.getID()+"u", vk.getID()+"u", vj.getID(), vk.getID(), t, Color.RED);
							e.setNodeOverlap(np.getA());
							g.addEdge(e, vj, vk);
						}
						
						if(np.getB() > 0)
						{
							float t = ((float)np.getB())*THICKNESS_INC + 1.0f;
							EdgeObj e = new EdgeObj(vj.getID()+"l", vk.getID()+"l", vj.getID(), vk.getID(), t, Color.BLUE);
							e.setNodeOverlap(np.getB());
							g.addEdge(e, vj, vk);
						}
					}
				}
			}
			layout.setInitializer(vv.getGraphLayout());
			layout.setRadius(300);
			layout.setGraph(g);
			vv.setGraphLayout(layout);
			
			frame = new JFrame();
			frame.setLayout(new BorderLayout());
			frame.setTitle("Community network: "+node.toString());
			frame.setSize(new Dimension(800,700));
			frame.setResizable(true);
			frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			frame.setLocationRelativeTo(null);
			frame.setJMenuBar(createMenuBar());

			Class[] combos = getCombos();
			final JComboBox jcb = new JComboBox(combos);
			jcb.setRenderer(new DefaultListCellRenderer() {
				static final long serialVersionUID = 1463;
	            public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
	                String valueString = value.toString();
	                valueString = valueString.substring(valueString.lastIndexOf('.')+1);
	                return super.getListCellRendererComponent(list, valueString, index, isSelected,
	                        cellHasFocus);
	            }
	        });
			jcb.addActionListener(new LayoutChooser(jcb, vv, g));
			jcb.setSelectedItem(CircleLayout.class);
			frame.add(jcb, BorderLayout.NORTH);
			
			JScrollPane pane = new JScrollPane();
			JScrollPane pane3 = new JScrollPane();
			mouseOverInfo = new JList();
			pane3.setViewportView(mouseOverInfo);

			JSplitPane pane2 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
					vv, pane3);
			pane2.setResizeWeight(0.9);
			frame.add(pane2, BorderLayout.CENTER);
			//frame.add(mouseOverInfo, BorderLayout.EAST);
			pane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
			pane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
			
			//pane.setViewportView(gzsp);
			frame.setVisible(true);
			frame.update(frame.getGraphics());
		}
	}
	
	private JMenuBar createMenuBar()
	{
		JMenuBar menuBar = new JMenuBar();
		
		JMenu fileMenu = new JMenu("File");
		
		String osName = System.getProperty("os.name");
		
		int mask_a = ActionEvent.CTRL_MASK;
		//int mask_b = ActionEvent.SHIFT_MASK;
		//int mask_c = ActionEvent.ALT_MASK;
		
		if (osName.startsWith("Mac OS"))
		{
			mask_a = ActionEvent.META_MASK;
		}
		
		fileMenu.setMnemonic(KeyEvent.VK_F);
		
		screenshot = new JMenuItem("Save community as eps...");
		screenshot.addActionListener(this);
		screenshot.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_P, mask_a ));
		fileMenu.addSeparator();
		fileMenu.add(screenshot);
		
		menuBar.add(fileMenu);
		
		return menuBar;
	}
	
	@SuppressWarnings("unchecked")
	private static Class[] getCombos()
    {
        List layouts = new ArrayList();
        layouts.add(KKLayout.class);
        layouts.add(FRLayout.class);
        layouts.add(CircleLayout.class);
        layouts.add(SpringLayout.class);
        layouts.add(ISOMLayout.class);
        return (Class[]) layouts.toArray(new Class[0]);
    }
	
	@SuppressWarnings("unchecked")
	private static final class LayoutChooser implements ActionListener
    {
        private final JComboBox jcb;
        private final VisualizationViewer<VertexObj, EdgeObj> vv;
        private SparseGraph<VertexObj, EdgeObj> g;

        private LayoutChooser(JComboBox jcb, VisualizationViewer<VertexObj, EdgeObj> vv, SparseGraph<VertexObj, EdgeObj> g)
        {
            super();
            this.jcb = jcb;
            this.vv = vv;
            this.g = g;
        }

        public void actionPerformed(ActionEvent arg0)
        {
        	Object[] constructorArgs = {g};
            Class layoutC = (Class) jcb.getSelectedItem();
            Class lay = layoutC;
            try
            {
                Constructor constructor = lay.getConstructor(new Class[] {Graph.class});
                Object o = constructor.newInstance(constructorArgs);
                Layout l = (Layout) o;
                l.setInitializer(vv.getGraphLayout());
                
                if(layoutC.toString().endsWith("SpringLayout"))
                {
                	SpringLayout sl = (SpringLayout)l;
                	sl.setForceMultiplier(0.5);
                	sl.setRepulsionRange(80);
                	sl.setStretch(0.5);
                	sl.setSize(new Dimension(600,600));
                	vv.setGraphLayout(sl);
                }
                else if(layoutC.toString().endsWith("FRLayout"))
                {
                	FRLayout frl = (FRLayout)l;
                	frl.setMaxIterations(500);
                	frl.setAttractionMultiplier(1.0);
                	frl.setRepulsionMultiplier(2.0);
                	frl.setSize(new Dimension(600,600));
                	vv.setGraphLayout(frl);
                }
                else if(layoutC.toString().endsWith("CircleLayout"))
                {
                	CircleLayout cl = (CircleLayout)l;
                	cl.setRadius(280);
                	vv.setGraphLayout(cl);
                }
                else if(layoutC.toString().endsWith("KKLayout"))
                {
                	KKLayout kkl = (KKLayout)l;
                	kkl.setMaxIterations(500);
                	kkl.setSize(new Dimension(600,600));
                	vv.setGraphLayout(kkl);
                }
                else if(layoutC.toString().endsWith("ISOMLayout"))
                {
                	ISOMLayout isoml = (ISOMLayout)l;
                	isoml.setSize(new Dimension(600,600));
                	vv.setGraphLayout(isoml);
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
    }

	private class CustomPickSupport extends ShapePickSupport<VertexObj, EdgeObj>
	{
		private BicliqueTreeMenu tm;
		private CustomVV<VertexObj, EdgeObj> cvv;
	    public CustomPickSupport(VisualizationServer<VertexObj, EdgeObj> vv, BicliqueTreeMenu t, CustomVV<VertexObj, EdgeObj> cvv) 
	    {
	        super(vv);
	        tm = t;
	        this.cvv = cvv;
	    }
	    
	    public VertexObj getVertex(Layout<VertexObj, EdgeObj> layout, double x, double y) 
	    {
	    	VertexObj closest = null;
	        double minDistance = Double.MAX_VALUE;
	        Point2D ip = vv.getRenderContext().getMultiLayerTransformer().inverseTransform(Layer.VIEW, new Point2D.Double(x,y));
	        x = ip.getX();
	        y = ip.getY();

	        while(true) 
	        {
	            try 
	            {
	                for(VertexObj v : getFilteredVertices(layout)) 
	                {
	                	
	                    Shape shape = vv.getRenderContext().getVertexShapeTransformer().transform(v);
	                    // get the vertex location
	                    Point2D p = layout.transform(v);
	                    if(p == null) continue;
	                    // transform the vertex location to screen coords
	                    p = vv.getRenderContext().getMultiLayerTransformer().transform(Layer.LAYOUT, p);
	                    
	                    double ox = x - p.getX();
	                    double oy = y - p.getY();

	                    if(shape.contains(ox, oy)) 
	                    {
	                    	
	                    	if(style == Style.LOWEST) 
	                    	{
	                    		// return the first match
	                    		return v;
	                    	} 
	                    	else if(style == Style.HIGHEST) 
	                    	{
	                    		// will return the last match
	                    		closest = v;
	                    	} 
	                    	else 
	                    	{
	                    		
	                    		// return the vertex closest to the
	                    		// center of a vertex shape
		                        Rectangle2D bounds = shape.getBounds2D();
		                        double dx = bounds.getCenterX() - ox;
		                        double dy = bounds.getCenterY() - oy;
		                        double dist = dx * dx + dy * dy;
		                        if (dist < minDistance) 
		                        {
		                        	minDistance = dist;
		                        	closest = v;
		                        }
	                    	}
	                    }
	                }
	                break;
	            } 
	            catch(ConcurrentModificationException cme) 
	            {
	            	
	            }
	        }
	        
	        if(closest != null)
	        {
	        	TreePath tp = tm.getJTree().getNextMatch(closest.toString(), selectedRow, Position.Bias.Forward);
	        	if(!cvv.getToolTipOn())
	        	{
	        		boolean allowed = true;
	        		DefaultMutableTreeNode n = (DefaultMutableTreeNode)tp.getLastPathComponent();
		        	Community c = (Community)n.getUserObject();
		        	
					if(c.getNumOfTotalNodes() >= BiCliqueWin.WARNING_NODESIZE)
					{
						int choice = JOptionPane.showConfirmDialog(frame,"Are you sure you wish\n to open the following community?\n" +
								"It contains more than "+BiCliqueWin.WARNING_NODESIZE+" nodes", "Large community", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
						
						if(choice == JOptionPane.NO_OPTION)
						{
							allowed = false;
						}
					}
					
					if(allowed)
					{
			        	tm.getJTree().setSelectionPath(tp);
			        	tm.getJTree().scrollPathToVisible(tp);
					}
	        	}
	        	DefaultMutableTreeNode n = (DefaultMutableTreeNode)tp.getLastPathComponent();
	        	Community c = (Community)n.getUserObject();
	        	Vector<String> vs = new Vector<String>();
	        	
	        	Iterator<Integer> it = c.getAllUpperNodes().iterator();
	        	
	        	vs.add("NODE A:");
	        	while(it.hasNext())
	        	{
	        		Integer id = it.next();
	        		
	        		vs.add(infoFile.getIDToInfo().get(new IDNodeTypePair(id, true)).toString());
	        	}
	        	vs.add("   ");
	        	vs.add("NODE B:");
	        	
	        	it = c.getAllLowerNodes().iterator();
	        	
	        	while(it.hasNext())
	        	{
	        		Integer id = it.next();
	        		
	        		vs.add(infoFile.getIDToInfo().get(new IDNodeTypePair(id, false)).toString());
	        	}
	        	
	        	mouseOverInfo.setListData(vs);
	        	mouseOverInfo.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), 
	        			"Comm. "+c.getID(), TitledBorder.LEFT, TitledBorder.TOP));
	        }
	        
	        return closest;
	    }
	    
	    public EdgeObj getEdge(Layout<VertexObj, EdgeObj> layout, double x, double y) 
	    {

	        Point2D ip = vv.getRenderContext().getMultiLayerTransformer().inverseTransform(Layer.VIEW, new Point2D.Double(x,y));
	        x = ip.getX();
	        y = ip.getY();

	        // as a Line has no area, we can't always use edgeshape.contains(point) so we
	        // make a small rectangular pickArea around the point and check if the
	        // edgeshape.intersects(pickArea)
	        Rectangle2D pickArea = 
	            new Rectangle2D.Float((float)x-pickSize/2,(float)y-pickSize/2,pickSize,pickSize);
	        EdgeObj closest = null;
	        double minDistance = Double.MAX_VALUE;
	        while(true) 
	        {
	            try 
	            {
	                for(EdgeObj e : getFilteredEdges(layout)) 
	                {

	                    Pair<VertexObj> pair = layout.getGraph().getEndpoints(e);
	                    VertexObj v1 = pair.getFirst();
	                    VertexObj v2 = pair.getSecond();
	                    boolean isLoop = v1.equals(v2);
	                    Point2D p1 = vv.getRenderContext().getMultiLayerTransformer().transform(Layer.LAYOUT, layout.transform(v1));
	                    Point2D p2 = vv.getRenderContext().getMultiLayerTransformer().transform(Layer.LAYOUT, layout.transform(v2));
	                    if(p1 == null || p2 == null) continue;
	                    float x1 = (float) p1.getX();
	                    float y1 = (float) p1.getY();
	                    float x2 = (float) p2.getX();
	                    float y2 = (float) p2.getY();

	                    // translate the edge to the starting vertex
	                    AffineTransform xform = AffineTransform.getTranslateInstance(x1, y1);

	                    Shape edgeShape = 
	                    	vv.getRenderContext().getEdgeShapeTransformer().transform(Context.<Graph<VertexObj, EdgeObj>,EdgeObj>getInstance(vv.getGraphLayout().getGraph(),e));
	                    if(isLoop) 
	                    {
	                        // make the loops proportional to the size of the vertex
	                        Shape s2 = vv.getRenderContext().getVertexShapeTransformer().transform(v2);
	                        Rectangle2D s2Bounds = s2.getBounds2D();
	                        xform.scale(s2Bounds.getWidth(),s2Bounds.getHeight());
	                        // move the loop so that the nadir is centered in the vertex
	                        xform.translate(0, -edgeShape.getBounds2D().getHeight()/2);
	                    } 
	                    else 
	                    {
	                        float dx = x2 - x1;
	                        float dy = y2 - y1;
	                        // rotate the edge to the angle between the vertices
	                        double theta = Math.atan2(dy,dx);
	                        xform.rotate(theta);
	                        // stretch the edge to span the distance between the vertices
	                        float dist = (float) Math.sqrt(dx*dx + dy*dy);
	                        xform.scale(dist, 1.0f);
	                    }

	                    // transform the edge to its location and dimensions
	                    edgeShape = xform.createTransformedShape(edgeShape);

	                    // because of the transform, the edgeShape is now a GeneralPath
	                    // see if this edge is the closest of any that intersect
	                    if(edgeShape.intersects(pickArea)) 
	                    {
	                        float cx=0;
	                        float cy=0;
	                        float[] f = new float[6];
	                        PathIterator pi = new GeneralPath(edgeShape).getPathIterator(null);
	                        if(pi.isDone()==false) 
	                        {
	                            pi.next();
	                            pi.currentSegment(f);
	                            cx = f[0];
	                            cy = f[1];
	                            if(pi.isDone()==false) 
	                            {
	                                pi.currentSegment(f);
	                                cx = f[0];
	                                cy = f[1];
	                            }
	                        }
	                        float dx = (float) (cx - x);
	                        float dy = (float) (cy - y);
	                        float dist = dx * dx + dy * dy;
	                        if (dist < minDistance) {
	                            minDistance = dist;
	                            closest = e;
	                        }
	                    }
			        }
			        break;
			    } 
	            catch(ConcurrentModificationException cme) {}
			}
	        
	        if(closest != null)
	        {
	        	Pair<VertexObj> pair = layout.getGraph().getEndpoints(closest);
                VertexObj v1 = pair.getFirst();
                VertexObj v2 = pair.getSecond();
                
                TreePath tp = tm.getJTree().getNextMatch(v1.toString(), selectedRow, Position.Bias.Forward);
                TreePath tp2 = tm.getJTree().getNextMatch(v2.toString(), selectedRow, Position.Bias.Forward);
                
                DefaultMutableTreeNode n = (DefaultMutableTreeNode)tp.getLastPathComponent();
                DefaultMutableTreeNode n2 = (DefaultMutableTreeNode)tp2.getLastPathComponent();
	        	Community c = (Community)n.getUserObject();
	        	Community c2 = (Community)n2.getUserObject();
	        	Vector<String> vs = new Vector<String>();
	        	
	        	HashSet<Integer>[] overlap = c.getOverlappingNodes(c2);
	        	String type = "";
	        	if(closest.getFill().equals(Color.RED))
	        	{
	        		Iterator<Integer> it = overlap[0].iterator();
	        		vs.add("NODE A:");
		        	while(it.hasNext())
		        	{
		        		Integer id = it.next();
		        		
		        		vs.add(infoFile.getIDToInfo().get(new IDNodeTypePair(id, true)).toString());
		        	}
		        	type = "(A)";
	        	}
	        	else
	        	{
	        		Iterator<Integer> it = overlap[1].iterator();
	        		vs.add("NODE B:");
		        	while(it.hasNext())
		        	{
		        		Integer id = it.next();
		        		
		        		vs.add(infoFile.getIDToInfo().get(new IDNodeTypePair(id, false)).toString());
		        	}
		        	type = "(b)";
	        	}
	        	
	        	mouseOverInfo.setListData(vs);
	        	mouseOverInfo.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK), 
	        			c.getID()+","+c2.getID()+type, TitledBorder.LEFT, TitledBorder.TOP));
	        }
	        
			return closest;
	    }
	}
	
	/**
	 * 
	 * Makes a screen dump of the specified component
	 * 
	 * @param c Component to make a screen dump of
	 */
	public void takeScreenshot(Component c)
	{
		Dump dumper = new EPSDump();
		final JFileChooser fc = new JFileChooser(System.getProperty("user.dir"));
		fc.setDialogTitle("Screenshot");
		int returnVal = fc.showSaveDialog(frame);
		
		if (returnVal == JFileChooser.APPROVE_OPTION)
		{
			try 
			{
				File f = new File(fc.getSelectedFile().getPath()+".eps");
				dumper.dumpComponent(f, c);
			} 
			catch (IOException e) 
			{
				e.printStackTrace();
			}
		}
	}
	
	public void actionPerformed(ActionEvent e)
	{
		Object o = e.getSource();
		
		if(o.equals(screenshot))
        {
        	takeScreenshot(vv);
        }
	}
}
